/********************************************************************************************************/
/*
 Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "Acquisition.h"
//
using namespace Zor::Tools;
using namespace Zor;
//
namespace AAC {
//
/*************************************************************************/
//
Acquisition::Acquisition(){
    _Device = new Device();                              // Initialisation du priphrique de controle de la capture
    _Analyzer = new Analyzer();
    _Controller = new Controller(_Analyzer);
}
//
/*************************************************************************/
//
Acquisition::~Acquisition(){
    if(_Controller) delete(_Controller);
    if(_Analyzer) delete(_Analyzer);
    if(_Device)   delete(_Device);    // Destruction du peripherique de controle FPS
}
//
/*************************************************************************/
//
void Acquisition::run(){
	// Recuperation du flux video dans une IplImage
	IplImage* iplimage;
	// Lancement du peripherique de capture
    _Device->start(iplimage);
    // Webcam en marche
    _WebcamState=MARCHE;
    // Mise  jour des informations de l'analyzer
	_Analyzer->update(_Device->getResolution());
	// Lancement du controlleur
    _Controller->start();
    // Boucle de parcourt pour rcuperer les images provenant de la webcam
	while((iplimage = cvQueryFrame(_Device->getCapture())) && _Device->getState()){
		// Recuperation de l'image du flux correcte ?
		if(iplimage){
			_Analyzer->analyze(iplimage);                            // Analyse de l'image
            cvWaitKey(40);                                           // Temporisation
		}
		// Arret de la capture ?
		_MutexState.lock();
        if(_WebcamState==ARRET) {
            _Controller->setState(ARRET);
            while(_Controller->isRunning()){}
            _Device->setState(false);
        }
        _MutexState.unlock();
	}
	// Relachement du controleur de la camera
	_Device->stop();
}
//
/*************************************************************************/
//
void Acquisition::setState(ETAT_WEBCAM etat){
    //
    _MutexState.lock();
    switch (_WebcamState) {
        case ARRET :
            if(etat==MARCHE) {
                _WebcamState=MARCHE;
                this->start();
            }
            break;
        case MARCHE :
            if(etat==ARRET) {
                _WebcamState=ARRET;
            }
            break;
        default :
            break;
    }
    _MutexState.unlock();
}
//
/*************************************************************************/
//
} // AAC
//
/*************************************************************************/
