/********************************************************************************************************/
/*
 Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frédéric (University of Science and Technology of Nantes)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "Analyzer.h"
//
using namespace GC::Tools;
//
namespace AAC {
//
/*************************************************************************/
//
Analyzer::Analyzer(){
}
//
/*************************************************************************/
//
Analyzer::~Analyzer(){
}
//
/*************************************************************************/
//
void Analyzer::calculDistanceXY(QVector<QPoint> point){
    // Re Initialisation des variables
    double d1, d2, d3;

    // Calcul du centre
	QPoint center( (point.at(0).x()+point.at(1).x()+point.at(2).x())/3,(point.at(0).y()+point.at(1).y()+point.at(2).y())/3 );

	// Calcul de la distance maximale pour X
	d1 = sqrt(pow( static_cast<double>(point.at(0).x() - center.x()) ,2));
	d2 = sqrt(pow( static_cast<double>(point.at(1).x() - center.x()) ,2));
	d3 = sqrt(pow( static_cast<double>(point.at(2).x() - center.x()) ,2));
	_DistanceXY.rx() = (d1+d2+d3)/3.0;
    //_DistanceXY.rx() = ( std::max(d3,std::max(d1,d2)) );

    // Calcul de la distance maximale pour Y
    d1 = sqrt(pow( static_cast<double>(point.at(0).y() - center.y()) ,2)) ;
    d2 = sqrt(pow( static_cast<double>(point.at(1).y() - center.y()) ,2)) ;
    d3 = sqrt(pow( static_cast<double>(point.at(2).y() - center.y()) ,2)) ;
    _DistanceXY.ry() = (d1+d2+d3)/3.0;
    //_DistanceXY.ry() = ( std::max(d3,std::max(d1,d2)) );

}
//
/*************************************************************************/
//
void Analyzer::calculRayonC(QVector<QPoint> point){
    // Re Initialisation des variables
    _RayonC.clear();
    double d1, d2, d3;

    // Calcul des rayons avec coefficient
    for(int i = 0; i<3; i++){
        point.value(i).rx() = static_cast<int>(point.at(i).x() * _Coefficient.x()); point.value(i).ry() = static_cast<int>(point.at(i).y() *  _Coefficient.y());
    }
	QPoint center( (point.at(0).x()+point.at(1).x()+point.at(2).x())/3,(point.at(0).y()+point.at(1).y()+point.at(2).y())/3 );
	d1 = sqrt(pow( static_cast<double>(point.at(0).x() - center.x()) ,2) + pow( static_cast<double>(point.at(0).y() - center.y()) ,2)) ;
	d2 = sqrt(pow( static_cast<double>(point.at(1).x() - center.x()) ,2) + pow( static_cast<double>(point.at(1).y() - center.y()) ,2)) ;
	d3 = sqrt(pow( static_cast<double>(point.at(2).x() - center.x()) ,2) + pow( static_cast<double>(point.at(2).y() - center.y()) ,2)) ;

    _RayonC.push_back((d1+d2+d3)/3.0 ) ;
    //_RayonC.push_back( std::max(d3,std::max(d1,d2)) ) ;
    _RayonC.push_back( _RayonC.at(0) * 0.50 *_Coefficient.x() );
    _RayonC.push_back( _RayonC.at(0) * 1.50 *_Coefficient.y() );
}
//
/*************************************************************************/
//
int Analyzer::getPoints(){
	//entrée en Section Critique
	_MutexState.lock();
	int nbPoints = _ListePoints.size();
	_ListePointsAttente.clear();
    for (int i=0;i<_ListePoints.size();i++){
		_ListePointsAttente.push_back(_ListePoints.at(i));
    }
	_MutexState.unlock();
	//sortie de Section Critique
	return nbPoints;
}
//
/*************************************************************************/
//
void Analyzer::setPoints(QVector<CvPoint> &liste){
	//entrée en Section Critique
	_MutexState.lock();
	if (liste.size() <= 3) {
	    _ListePoints.clear();
		for (int i=0;i<liste.size();i++){
			_ListePoints.push_back(liste.at(i));
        }
	}
	//sortie de Section Critique
	_MutexState.unlock();

}
//
/*************************************************************************/
//
void Analyzer::update(const QPoint& point){
    // Récuperation de la résolution de la webcam
    _ResolutionWebcam = point;

    // Récupération du seuillage
    _Seuil.clear();
    _Seuil = Divers::getPoints("calibrage/calibrageCouleurs.gc");

    // Calcul du rayon sans coefficient
    calculDistanceXY(Divers::getPoints("calibrage/calibrageMouvement.gc"));

    // Recupération de la valeur de tolérance
    _Decalage = QPointF(_DistanceXY.x() * (1.5+0.5), _DistanceXY.y() * (1.5+0.5));
   // _Tolerance = QPointF(_DistanceXY.x() * 2.5, _DistanceXY.y() * 2.5);


    // Calcul du coefficient de deplacement
    QDesktopWidget ecran;
    _ResolutionEcran = QPoint(ecran.screen()->width(),ecran.screen()->height());
    if((_ResolutionWebcam.x()==0) || (_ResolutionWebcam.y()==0)){
        _Coefficient = QPointF(1.0 ,1.0);
    } else {
        _Coefficient = QPointF((float)_ResolutionEcran.x() / (_ResolutionWebcam.x()*0.7/*- _Tolerance.x()*/) , (float)_ResolutionEcran.y() / (_ResolutionWebcam.y()*0.7/*-  _Tolerance.y()*/));
    }

    // Calcul du rayon avec coefficient
    calculRayonC(Divers::getPoints("calibrage/calibrageMouvement.gc"));

    std::cout << "Resolution Ecran : " <<  _ResolutionEcran.x() << " " <<  _ResolutionEcran.y() << std::endl;
    std::cout << "Resolution Webcam : " <<  _ResolutionWebcam.x() << " " <<  _ResolutionWebcam.y() << std::endl;
    std::cout << "Coefficient : " <<  _Coefficient.x() << " " <<  _Coefficient.y() << std::endl;
    std::cout << "Zone Admissible : " << _DistanceXY.x() << " " << _DistanceXY.y() << std::endl;
}
//
/*************************************************************************/
//
void Analyzer::analyze(IplImage* image)
	{
		IplImage * image_work;
		IplImage * masque;
		IplImage * image_fin;
		IplImage * image_sortie_rects;



		masque = cvCreateImage(cvGetSize(image),IPL_DEPTH_8U, 1);
		cvCvtColor(image,masque,CV_BGR2GRAY);
		// application d'un seuillage pour trouer les zones blanches du masque
		cvThreshold(masque,masque,120,255,CV_THRESH_BINARY_INV);
		cvErode(masque,masque,NULL,1);

		// création de l'image sur laquelle le seuillage sera appliqué
		image_work = cvCreateImage(cvGetSize(image),IPL_DEPTH_8U, 1);
		cvSmooth(image_work,image_work,CV_GAUSSIAN,3,3);
		cvDilate(image_work,image_work,NULL,3);


		cvInRangeS(image,CV_RGB(_Seuil.at(0).x(),_Seuil.at(1).x(),_Seuil.at(2).x()),CV_RGB(_Seuil.at(0).y()+1,_Seuil.at(1).y()+1,_Seuil.at(2).y()+1),image_work);

		cvDilate(image_work,image_work,NULL,6);
		cvErode(image_work,image_work,NULL,3);

		image_fin = cvCreateImage(cvGetSize(image),IPL_DEPTH_8U, 1);
		cvSub(image_work,masque,image_fin,NULL);
		cvSmooth(image_fin,image_fin,CV_GAUSSIAN,3,3);
		//cvShowImage("TEST",image_fin);

		cvReleaseImage(&image_work);
		cvReleaseImage(&masque);

		//Find the contours of the moving images in the frame.
		CvMemStorage* storage = cvCreateMemStorage(0);
		CvSeq* contour = 0;

		//Rectangle to use to put around the people.
		CvRect bndRect = cvRect(0,0,0,0);

		//Points for the edges of the rectangle.
		CvPoint pt1, pt2, pti1;//pti2;

		cvFindContours( image_fin, storage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );

		cvReleaseImage(&image_fin);

		image_sortie_rects = cvCreateImage(cvGetSize(image),IPL_DEPTH_8U, 3);
		//Process each moving contour in the current frame...
		QVector<CvPoint> valeurs;

		for( ; contour != 0; contour = contour->h_next )
		{
			bndRect = cvBoundingRect(contour, 0);
			pt1.x = bndRect.x;
			pt1.y = bndRect.y;
			pt2.x = bndRect.x + bndRect.width;
			pt2.y = bndRect.y + bndRect.height;

			cvRectangle(image_sortie_rects, pt1, pt2, CV_RGB(255,0,0), 3);
            #ifdef WIN32
			pti1.x = ((_ResolutionWebcam.x()-(pt1.x+((pt2.x-pt1.x)/2)))- _Decalage.x() )*_Coefficient.x();
			pti1.y = ((_ResolutionWebcam.y()-(pt2.y+((pt2.y-pt1.y)/2)))-  _Decalage.y() )*_Coefficient.y();
			#else
			pti1.x = ((_ResolutionWebcam.x()-(pt1.x+((pt2.x-pt1.x)/2)))- _Decalage.x() )*_Coefficient.x();
			pti1.y = (((pt2.y-60+((pt2.y-pt1.y)/2)))-  _Decalage.y() )*_Coefficient.y();
			#endif
			if(pti1.x<0) { pti1.x=0; } else if(pti1.x>_ResolutionEcran.x()) { pti1.x = _ResolutionEcran.x(); }
			if(pti1.y<0) { pti1.y=0; } else if(pti1.y>_ResolutionEcran.y()) { pti1.y = _ResolutionEcran.y(); }


			valeurs.append(pti1);
		}

		cvReleaseMemStorage(&storage);
		cvReleaseImage(&image_sortie_rects);

		//mise à  jour de la liste des points
		setPoints(valeurs);
	}
	//
/*************************************************************************/
//
} // AAC
//
/*************************************************************************/



