/********************************************************************************************************/
/* Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frédéric (University of Science and Technology of Nantes)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
// NOTE : Utilité du controller :
// le controller est un thread séparer qui consulte les données stockées par
// l'analyseur d'images. Il se sert de ces données pour créer les états
// correspondants. C'est lui qui selon le cas, en créant (ou changeant) les états permet
// de générer les événements système. Notez que les appels système sont générés par les états.
//
/*************************************************************************/
//
#include "Controller.h"
//
using namespace GC::Tools;
using namespace Zor::Tools;
//
namespace AAC {
//
/*************************************************************************/
//Constructeur
Controller::Controller(Analyzer * analyzer){
	_Analyseur = analyzer;


}
/*************************************************************************/
//Destructeur
Controller::~Controller() {
}
/*************************************************************************/
//void Controller::setState(Etat * newState){
//		_state = newState;
//}
/*************************************************************************/
// Méthode appelée en boucle permettant de construire les états
// Le controleur consulte les données de l'analyseur, et change l'état
// du système.
void Controller::handle_case(){

	int nb_points = _Analyseur->getPoints() ;

	//QTest::qSleep(5000);
	//nb_points=3;
	switch (nb_points){
		case 1 :
			recuperePoints();
			_state = EE::Etat1::getInstance() ;
			_state->execute(this) ;
			break;
		case 2 :

			recuperePoints();
			_state = EE::Etat2::getInstance() ;
			_state->execute(this) ;
			break;

		case 3 :
			recuperePoints();

			_state = EE::Etat3::getInstance() ;
			_state->execute(this) ;
			break;

		default :
			break;
	}

}
//
/*************************************************************************/
// Méthode de consultation des données de l'analyseur d'image
// Cette méthode récupère les points trouvés, et renvoie leur nombre.
void Controller::recuperePoints(){
    _ListePoints = _Analyseur->getListePoints();
}
//
/*************************************************************************/
//
void Controller::run(){
    QVector<double> rayon = _Analyseur->getRayonC();
    _Rayon = rayon.at(0);
    _RayonGauche = rayon.at(1);
    _RayonDroit = rayon.at(2);
    //std::cout << "Rayon : " << rayon << std::endl;
    std::cout << "Rayon Gauche : " << _RayonGauche << std::endl;
    std::cout << "Rayon Droit : " << _RayonDroit << std::endl;
    _WebcamState=MARCHE;
	while(true){
		handle_case();
        QTest::qSleep(10);
		_MutexState.lock();
		if(_WebcamState==ARRET) {
            _MutexState.unlock();
            break;
        }
        _MutexState.unlock();
	}
}
//
/*************************************************************************/
//
void Controller::setState(ETAT_WEBCAM etat){
    //
    _MutexState.lock();
    switch (_WebcamState) {
        case ARRET :
            if(etat==MARCHE) {
                _WebcamState=MARCHE;
                this->start();
            }
            break;
        case MARCHE :
            if(etat==ARRET) {
                _WebcamState=ARRET;
            }
            break;
        default :
            break;
    }
    _MutexState.unlock();
}
//
/*************************************************************************/
//
} // AAC
//
/*************************************************************************/
//
