/********************************************************************************************************/
/*
 Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frédéric (University of Science and Technology of Nantes)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Contact: glovecontrol@free.fr
 */
/********************************************************************************************************/
//
#include "Etat.h"
//
using namespace AAC;
//
namespace AAC {
namespace EE {
//
/**CLASSE ABSTRAITE ETAT***************************************************/
bool Etat::_LeftDown = false;
bool Etat::_RightDown = false;
Etat::~Etat() {
	//Destructeur de la super classe Etat
}
//
/*************************************************************************/
//

//
/*************************************************************************/
/***ETAT1*****************************************************************/
//
Etat1 * Etat1::_instance = 0;
Etat1::Etat1(){}
Etat1::~Etat1(){}
//
Etat1 * Etat1::getInstance() {
	if (_instance == 0) {
		_instance = new Etat1() ;
	}
	return _instance ;
}
//
void Etat1::execute(Controller * sender){
    //Events::DeplaceCurseurPt(calculCentre(sender) ) ;
}
//
//évaluée du curseur
CvPoint Etat1::calculCentre(Controller * sender){
	CvPoint pt;
	pt.x = (sender->getListePoints().at(0).x);
	pt.y = (sender->getListePoints().at(0).y);
	return pt;
}
/*************************************************************************/
/***ETAT2*****************************************************************/
//
Etat2 * Etat2::_instance = 0;
Etat2::Etat2(){}
Etat2::~Etat2(){}
//
/*************************************************************************/
//
Etat2 * Etat2::getInstance() {
	if (_instance == 0) {
		_instance = new Etat2() ;
	}
	return _instance ;
}
//
void Etat2::execute(Controller * sender){
   // Events::DeplaceCurseurPt(calculCentre(sender) ) ;
}
//
/*************************************************************************/
//fait la moyenne des coordonnées et retourne la position
//évaluée du curseur
CvPoint Etat2::calculCentre(Controller * sender){
	CvPoint pt;
	pt.x = (sender->getListePoints().at(0).x+sender->getListePoints().at(1).x)/2;
	pt.y = (sender->getListePoints().at(0).y+sender->getListePoints().at(1).y)/2;
	return pt;
}
//
/*************************************************************************/
/***ETAT3*****************************************************************/
//
Etat3 * Etat3::_instance = 0;
Etat3::Etat3(){
    maxPoints = 40;
    timeGauche.start();
    timeDroit.start();
}
Etat3::~Etat3(){}
//
Etat3 * Etat3::getInstance() {
	if (_instance == 0) {
		_instance = new Etat3() ;
	}
	return _instance ;
}
//
void Etat3::execute (Controller * sender){
	int leray;
    int temporisation = 1500;
	bool existeCliqueGauche = (timeGauche.elapsed()<temporisation);
	bool existeCliqueDroit =  (timeDroit.elapsed()<temporisation);


    updateListePoints(calculCentre(sender));
    actualCenter = moyenneListePoints();


	leray = rayon(sender);

    //std::cout << "Temps Ecoule : " << time.elapsed() << std::endl;
	switch(leray){

		case 0 :
            // Relachement du clique droit [enfonce]
            if(_RightDown) {
                std::cout << "Clique Droit [ RELACHE ]" << std::endl;
                Events::deplaceCurseurPt(actualCenter) ;
                Events::clickDroitUp(actualCenter);
                _RightDown = false;
            }
            if(!existeCliqueDroit){
                // Clique gauche ?
                //std::cout << "timeGauche : " << timeGauche.elapsed() << "bool : " << existeCliqueGauche << std::endl;
                if (!_LeftDown){
                    // Pas encore de clique gauche alors clique gauche ou double clique gauche
                    _LeftDown=true;
                    // Clique gauche ou double clique ?
                    if(existeCliqueGauche){
                        // Double clique
                        std::cout << "Clique Gauche [ DOUBLECLICK ENFONCE ]" << std::endl;
                        Events::deplaceCurseurPt(lastCenter) ;
                        Events::doubleClick(lastCenter) ;
                    } else {
                        // Simple clique
                        timeGauche.restart();
                        lastCenter = actualCenter;
                        Events::deplaceCurseurPt(actualCenter) ;
                        Events::clickGaucheDown(actualCenter) ;
                        std::cout << "Clique Gauche [ ENFONCE ]" << std::endl;
                    }
                } else {
                    if(!(existeCliqueGauche)){
                        //std::cout << "Clique Gauche [ DEPLACEMENT ]" << std::endl;
                        Events::glisseCurseurGauchePt(actualCenter);
                    }
                }
            }
			break;
		case 1 :
            // Relachement du clique gauche [enfonce]
			if (_LeftDown){
			    std::cout << "Clique Gauche [ RELACHE ]" << std::endl;
			    if(existeCliqueGauche) {
                    Events::deplaceCurseurPt(lastCenter) ;
                    Events::clickGaucheUp(lastCenter) ;
			    } else {
                    Events::deplaceCurseurPt(actualCenter) ;
                    Events::clickGaucheUp(actualCenter) ;
			    }
                _LeftDown=false;
			}
			// Relachement du clique droit [enfonce]
			if (_RightDown){
			    std::cout << "Clique Droit [ RELACHE ]" << std::endl;
			    if(existeCliqueDroit) {
                    Events::deplaceCurseurPt(lastCenter) ;
                    Events::clickDroitUp(lastCenter) ;
			    } else {
                    Events::deplaceCurseurPt(actualCenter) ;
                    Events::clickDroitUp(actualCenter) ;
			    }
                _RightDown=false;
			}
			if( !(existeCliqueGauche) &&  !(existeCliqueDroit) ){
                //std::cout << "Clique Droit [ MVT ]" << std::endl;
                Events::deplaceCurseurPt(actualCenter) ;
            }
			break;
		case 2 :
            // Relachement du clique gauche [enfonce]
			if (_LeftDown){
			    std::cout << "Clique Gauche [ RELACHE ]" << std::endl;
                Events::deplaceCurseurPt(actualCenter) ;
                Events::clickGaucheUp(actualCenter) ;
                _LeftDown=false;
			}
			if(!existeCliqueGauche){
                //std::cout << "timeDroit : " << timeDroit.elapsed() << "bool : " << existeCliqueDroit << std::endl;
                if (!_RightDown){
                    _RightDown=true;
                    timeDroit.restart();
                    std::cout << "Clique Droit [ ENFONCE ]" << std::endl;
                    lastCenter = actualCenter;
                    Events::deplaceCurseurPt(actualCenter) ;
                    Events::clickDroitDown(actualCenter) ;
                } else {
                    if(!(existeCliqueDroit)){
                        std::cout << "Clique Droit [ DEPLACEMENT ]" << std::endl;
                        Events::glisseCurseurDroitPt(actualCenter);
                    }
                }
			}
			break;
	}

}
//
/*************************************************************************/
//fait la moyenne des coordonnées et retourne la position
//évaluée du curseur
CvPoint Etat3::calculCentre(Controller * sender){
	CvPoint pt;
	pt.x = (sender->getListePoints().at(0).x+sender->getListePoints().at(1).x+sender->getListePoints().at(2).x)/3;
	pt.y = (sender->getListePoints().at(0).y+sender->getListePoints().at(1).y+sender->getListePoints().at(2).y)/3;
	return pt;
}
//
/*************************************************************************/
//
int Etat3::rayon(Controller * sender){
	double ray;

	ray=calcul_rayon(sender);
	std::cout << "Rayon Courant : " << ray << std::endl;
	if (ray<=sender->getRayonGauche()){
		return 0;
	}
	else if (ray>=sender->getRayonDroit()){
		return 2;
	}
		return 1;
}
//
/*************************************************************************/
//
double Etat3::calcul_rayon(Controller * sender){
	CvPoint pA,pB,pC,center;
	double d1,d2,d3;

	pA=sender->getListePoints().at(0);
	pB=sender->getListePoints().at(1);
	pC=sender->getListePoints().at(2);
	center = calculCentre(sender);
	/*
	std::cout << "centre des diodes   : " << center.x << " " << center.y << std::endl;
	std::cout << "centre des diodes 1 : " << pA.x << " " << pA.y << std::endl;
	std::cout << "centre des diodes 2 : " << pB.x << " " << pB.y << std::endl;
	std::cout << "centre des diodes 3 : " << pC.x << " " << pC.y << std::endl;
	*/
	d1= sqrt(pow( pA.x - center.x ,2) + pow( pA.y - center.y ,2)) ;
	d2= sqrt(pow( pB.x - center.x ,2) + pow( pB.y - center.y ,2)) ;
	d3= sqrt(pow( pC.x - center.x ,2) + pow( pC.y - center.y ,2)) ;

    return (d1+d2+d3)/3.0;
	//return std::max(d3,std::max(d1,d2));
}
//
/*************************************************************************/
//
void Etat3::updateListePoints(const CvPoint& point){
    for(int i=listePoints.size(); i<=maxPoints;i++){
        listePoints.push_front(point);
    }
    listePoints.pop_back();

}
//
/*************************************************************************/
//
CvPoint Etat3::moyenneListePoints(){
    CvPoint moyenne;
    moyenne.x = 0;
    moyenne.y = 0;
    for(int i=0; i<listePoints.size(); i++){
        moyenne.x += listePoints.at(i).x;
        moyenne.y += listePoints.at(i).y;
    }
    if(listePoints.size()!=0){
        moyenne.x /= listePoints.size();
        moyenne.y /= listePoints.size();
    }
    return moyenne;
}
//
/*************************************************************************/
//
} // EE
} // AAC
//
/*************************************************************************/
