/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
/*
Cette classe permet de d'initialiser la capture des images de la webcam. Elle permet galement de
rcuperer les rsolutions des images et le nombre de frames par seconde.
*/
//
#ifndef Device_H
#define Device_H
//
// Librairies Qt
#include <QTime>
#include <QPoint>
#include <QString>
// Librairies OpenCV
#ifdef WIN32 // l'ordinateur est sous windows
#include <cv.h>
#include <highgui.h>
#else // Ordinateur sous autre chose que windows
#include <OpenCV/cv.h>
#include <OpenCV/highgui.h>
#endif  // endif
// Librairies autres
#include <iostream>
//
namespace Zor {
//
class Device
{
	//
public:
	//
	Device();
	~Device();
	//
	void start(IplImage * iplimage);                            // Initialisation de la webcam
	void stop();                                                // Arrt de la webcam
	inline bool getState() { return _State; }                   // Rcuprer l'etat actuel de la webcam
	inline void setState(const bool& state) { _State=state; }   // Mettre  jour l'tat de la webcam
	inline CvCapture * getCapture() { return capture; }         // Rcuprer le prphrique de capture
	void update();                                              // Mettre  jour les informations (fps)
	QString getQStringFPS();                                    // Rcuprer les fps
	int getFPS();                                               // Rcuprer les fps
	QString getQStringResolution();                             // Rcuprer la rsolution
	QPoint getResolution();                                     // Rcuprer la rsolution
	//
private:
	//
	int currentFPS;                             // Nombre de fps actuel
	int lastFPS;                                // Variable temporaire
	int FPS;                                    // Variable temporaire
	bool _State;                                // Etat de la webcam
	QString _QString;                           // Chaine de caractere
	QTime * currentTime;                        // Compteur temps
	CvCapture * capture;                        // Peripherique pour capturer les frames via opencv
	double width;                               // Rsolution en largeur
	double height;                              // Rsolution en longueur
	//
};
//
}   // Zor
//
#endif // Device_H
//
/*************************************************************************************************************/
