/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
/*
Cette classe permet d'afficher des images dans le QPainter et de faire des oprations de rotation pour remettre
l'image dans le bon sens
*/
#ifndef Display_H
#define Display_H
//
// Librairies QT
#include <QtGui/QWidget>
#include <QPainter>
#include <QTest>
#include <QThread>
#include <QMouseEvent>
#include <QRectF>
#include <QMutex>
// Librairies OpenCV
#ifdef WIN32 // l'ordinateur est sous windows
#include <cv.h>
#include <highgui.h>
#else // Ordinateur sous autre chose que windows
#include <OpenCV/cv.h>
#include <OpenCV/highgui.h>
#endif  // endif
// Librairies Zor
#include "Tools.h"
// Librairies Autres
#include <iostream>

//
namespace Zor {
//
class Display : public QWidget {
    //
	Q_OBJECT
	//
public:
    //
	Display(QWidget * parent);
	~Display();
	//
	void updateQImage(IplImage * iplimage);                 // Mise  jour de l'image en prenant une IplImage en paramtre
	void updateQImage(QImage * qimage);                     // Mise  jour de l'image en prenant une QImage en paramtre
	//
	QImage * _QImage;
	QImage * _DisplayQImage;
	Zor::Tools::TYPE_CONVERSION rotationImage;
	//
signals:
    //
    void existe(bool);                                      // Existance des zones
    //
public slots:
    //
    void switchRotation();                                  // Rotation de l'image
    //
protected:
    //
    virtual void paintEvent( QPaintEvent * event );         // Rafraichissement des images du widget
	virtual void mousePressEvent(QMouseEvent *event);       // Evnement souris (touche presse)
   	virtual void mouseMoveEvent(QMouseEvent *event);        // Evnement souris (mouvement)
   	virtual void mouseReleaseEvent(QMouseEvent *event);     // Evnement souris (touche relache)
    //
    QMutex _Mutex;                                          // Mutex qui protge les zones critiques
    //
};
//
}
//
#endif // Display_H
//
/********************************************************************************************************/
