/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "DisplayCouleurs.h"
//
using namespace Zor::Tools;
//
namespace Zor{
//
/*************************************************************************/
// Constructeur de la classe (initialisation)
DisplayCouleurs::DisplayCouleurs(QWidget * parent): Display(parent) {
    for(int i=0; i<3; i++){
        _Element.push_back(new ZorRect(QRect(0, 0, 0, 0), false, false, false));
    }
}
//
/*************************************************************************/
// Destructeur de la classe
DisplayCouleurs::~DisplayCouleurs() {
    _Element.clear();
}
//
/*************************************************************************/
//
void DisplayCouleurs::paintEvent( QPaintEvent * event ) {
    QPainter painter(this);                                     // Priphrique de peinture
    painter.setBrush(QBrush(QColor(255,255,255)));              // Brosse de couleur blanche

    _Mutex.lock();                                              // Section critique ON
    // Affichage des images
    if(_DisplayQImage) {
        painter.drawImage(QPoint(0, 0), *_DisplayQImage);
    }
    _Mutex.unlock();                                            // Section critique OFF

    // Affichage des cercles
    bool allExist=true;
    for(int i=0; i<3; i++){
        painter.drawRect(_Element.at(i)->rectangle());
        allExist=(allExist && _Element.at(i)->existe());
    }
    // Image prsente en mmoire et toutes les cercles sont prsents
    if(_QImage && allExist) { emit existe(allExist); }

    painter.end();                                              // Fermeture du priphrique de peinture
}
//
/*************************************************************************/
//
void DisplayCouleurs::mousePressEvent(QMouseEvent *event) {
    // Selon le bouton
    switch(event->button()){
        case Qt::LeftButton:
            // Bouton gauche alors affichage d'un dbut de zone
            for(int i=0; i<3; i++){
                if(!(_Element.at(i)->existe())){
                    _Element.at(i)->rectangle().setCoords(event->x(),event->y(),event->x(),event->y());
                    _Element.at(i)->traite()=true;
                    break;
                }
            }
        break;
        case Qt::RightButton:
            // Bouton droit alors suppression de la zone sous le pointeur de la souris sinon rien
            for(int i=0; i<3; i++){
                if(_Element.at(i)->existe()){
                    if(_Element.at(i)->rectangle().contains(event->x(),event->y())){
                        _Element.at(i)->rectangle().setCoords(0, 0, 0, 0);
                        _Element.at(i)->existe()=false;
                        break;
                    }
                }
            }
        break;
        default:
        break;
    }
    // Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
void DisplayCouleurs::mouseReleaseEvent(QMouseEvent *event){
    // Relachement du bouton
    bool intersected;
    for(int i=0; i<3; i++){
        // Si l'element est en cours de traitement est qu'il ne rentre pas en collision avec une autre zone alors il apparat
        if(_Element.at(i)->traite()){
            intersected=false;
            for(int j=0; j<3; j++){
                if(j!=i) { intersected=intersected || _Element.at(i)->rectangle().intersects(_Element[j]->rectangle()); }
            }
            if(intersected){
                _Element.at(i)->rectangle().setCoords(0, 0, 0, 0);
            } else {
                _Element.at(i)->rectangle() = _Element.at(i)->rectangle().intersected(this->rect());
                _Element.at(i)->existe()=true;
                _Element.at(i)->traite()=false;
            }
            break;
        }
    }
    // Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
void DisplayCouleurs::mouseMoveEvent(QMouseEvent *event){
    // Si il y a un mouvement de la souris + bouton gauche enfonc alors actualisation de la dimension de la zone en temps rl
	if(event->buttons()==Qt::LeftButton){
        for(int i=0; i<3; i++){
            if(_Element.at(i)->traite()){
                _Element.at(i)->rectangle().setCoords(_Element.at(i)->rectangle().left(),_Element.at(i)->rectangle().top(),event->x(),event->y());
                break;
            }
        }
	}
	// Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
void DisplayCouleurs::effaceElements(){
    // Effacement de tous les lments
    for(int i=0; i<3; i++){
        _Element.at(i)->existe()=false;
        _Element.at(i)->rectangle().setCoords(0, 0, 0, 0);
    }
    // Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
}
//

