/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#include "DisplayMouvement.h"
//
using namespace Zor::Tools;
//
namespace Zor{
//
/*************************************************************************/
// Constructeur de la classe (initialisation)
DisplayMouvement::DisplayMouvement(QWidget * parent) : Display(parent) {
    for(int i=0; i<3; i++){
        _Element.push_back(new ZorRect(QRect(0, 0, 0, 0), false, false, false));
    }
}
//
/*************************************************************************/
// Destructeur de la classe
DisplayMouvement::~DisplayMouvement() {
    _Element.clear();
}
//
/*************************************************************************/
//
void DisplayMouvement::paintEvent( QPaintEvent * event ) {
    QPainter painter(this);                                     // Priphrique de peinture
    painter.setBrush(QBrush(QColor(255,255,255)));              // Brosse de couleur blanche

    _Mutex.lock();                                              // Section critique ON
    // Affichage des images
    if(_DisplayQImage) {
        painter.drawImage(QPoint(0, 0), *_DisplayQImage);
    }
    _Mutex.unlock();                                            // Section critique OFF

    // Affichage des cercles
    bool allExist=true;
    for(int i=0; i<3; i++){
        painter.drawEllipse(_Element.at(i)->rectangle());
        allExist=(allExist && _Element.at(i)->existe());
    }
    // Image prsente en mmoire et toutes les cercles sont prsents
    if(_QImage && allExist) { emit existe(allExist); }

    painter.end();                                              // Fermeture du priphrique de peinture
}
//
/*************************************************************************/
//
void DisplayMouvement::mousePressEvent(QMouseEvent *event) {
    // Selon le bouton
    switch(event->button()){
        case Qt::LeftButton:
            // Bouton gauche alors affichage des cercles
            for(int i=0; i<3; i++){
                if(!(_Element.at(i)->existe())){
                    _Element.at(i)->rectangle().setCoords(event->x()-5,event->y()-5,event->x()+10,event->y()+10);
                    _Element.at(i)->traite()=true;
                    break;
                }
            }
        break;
        case Qt::RightButton:
            // Bouton droit alors suppression du cercle pointer si il y en a un
            for(int i=0; i<3; i++){
                if(_Element.at(i)->existe()){
                    if(_Element.at(i)->rectangle().contains(event->x(),event->y())){
                        _Element.at(i)->rectangle().setCoords(0, 0, 0, 0);
                        _Element.at(i)->existe()=false;
                        break;
                    }
                }
            }
        break;
        default:
        break;
    }
    // Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
void DisplayMouvement::mouseReleaseEvent(QMouseEvent *event){
    // Bouton relch et une zone est trait alors un cercle est valide -> apparition
    for(int i=0; i<3; i++){
        if(_Element.at(i)->traite()){
            _Element.at(i)->existe()=true;
            _Element.at(i)->traite()=false;
            break;
        }
    }
    // Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
void DisplayMouvement::effaceElements(){
    // Effacement de tous les lments
    for(int i=0; i<3; i++){
        _Element.at(i)->existe()=false;
        _Element.at(i)->rectangle().setCoords(0, 0, 0, 0);
    }
    // Mise  jour de l'affichage
    this->update();
}
//
/*************************************************************************/
//
}
//

