/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
//
#ifndef ZorTools_H
#define ZorTools_H
//
// Librairies Qt
#include <QPoint>
#include <QString>
#include <QColor>
#include <QRectF>
#include <QtGui/QImage>
#include <QColor>
#include <QSettings>
#include <QDesktopWidget>
// Librairies Blitz++
#include <blitz/array.h>
// Librairies autres
#include "../Tools/OCVImage.h"
//
namespace Zor {
namespace Tools {
//
enum ETAT_INDIVIDU { NORMAL = 0, ELITE = 1, MUTEE = 2, CROISEE = 3 };       // Etat d'un gne
enum ETAT_WEBCAM { MARCHE=0, PAUSE=1, ARRET=2 };                        // Etat possible de la webcam
enum TYPE_CONVERSION { NO_INVERSION=0, INVERSION=1 };                   // Rotation
enum TYPE_HEURISTIQUE { HEU_0=0, HEU_1=1 };                   // Etat possible de la webcam
//
class ZorRect {
public:
    //
    ZorRect(const QRect& rectangle, const bool& existe, const bool& traite, const bool& selectionne);
    ~ZorRect();
    QRect& rectangle();
    bool& existe();
    bool& traite();
    bool& selectionne();
    //
private:
    //
    QRect _Rectangle;
    bool _Existe;
    bool _Traite;
    bool _Selectionne;
}; // Rect
//
class Individu {
public:
    QPoint rgb[3];  // Trois seuils (rouge, vert et bleu) possdant deux bornes (infrieur et suprieur)
    double valeur;  // Valeur du gne
    ETAT_INDIVIDU etat; // Etat du gne
}; // Individu
//
class Divers {
public:
    //
	virtual ~Divers();
	//

	static double calculRayonSelonResolution(const QRect& rect1, const QRect& rect2, const QRect& rect3, const QPoint& resolution);
    static int aleatoire(const int& min, const int& max);       // Gnration d'un nombre alatoire entre deux valeurs (min <= aleatoire < max)
    //
	static bool dansSeuil(const QColor& couleur, const Individu& filtre); // La couleur respecte le filtre
	static void triRapide(blitz::Array<Individu,1>& tableauEntree, const QPoint& zoneEntree);     // Tri rapide dans une zone (min <= zone < max)
    static int partitionner(blitz::Array<Individu,1>& tableauEntree, const QPoint& zoneEntree);   // Composante du tri-rapide
    static QImage * convertionIplImageQImage(IplImage * image, const TYPE_CONVERSION& type );
}; // Divers
//
} // ::Tools
} // ::Zor
#endif // ZorTools_H
//
/********************************************************************************************************/
