/********************************************************************************************************/
/*
Copyright (C) 2008 Biger Kevin, Marguet Quentin, Muller Thomas, Vinatier Frdric (University of Science and Technology of Nantes)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

Contact: glovecontrol@free.fr
*/
/********************************************************************************************************/
/* Notes (25/05/08)
L'algorithme peut produire des seuils ngatifs, cela ne pose pas de problme mais peut dtriorer la vitesse d'obtention des rsultats.
Correction : il faut crer des intervals de mutation (3) qui dpendent des intervals RGB (3).
La seconde heuristique n'est pas "stable" ni "optimis".
Correction : ajouter des try pour viter de planter sur une division par 0 et faire des tests pour ajuster les valeurs de l'heuristique.
L'algorithme ne possde pas de terminaison propre.
Correction : il y a diffrentes manires de faire terminer le programme (reboot, consenguinit...)
*/
/********************************************************************************************************/
//
#include "Seuil.h"
//
using namespace blitz;
using namespace Zor::Tools;
//
namespace Zor{
//
/********************************************************************************************************/
//
Seuil::Seuil( DisplayCouleurs * display, const TYPE_HEURISTIQUE& heuristique):_Display(display),_Heuristique(heuristique) {
}
//
/********************************************************************************************************/
//
Seuil::~Seuil() {
}
//
/********************************************************************************************************/
//
void Seuil::run() {
    // Rcupration de l'image
    _QImage=_Display->_QImage;
    _ImageSeuil=QImage(*_QImage);

    // Initialisation de l'alatoire + variables de parcourt
    srand(time(NULL));
    int x,y,i,j,z;

    // Rcupration des informations et redimmensionnement des zones
    float ratio_X = float(_QImage->width())/float(_Display->width());
    float ratio_Y = float(_QImage->height())/float(_Display->height());

    // Creation des 4 zones + Remise  l'chelle des zones + Initialisation de variables
    int zoneInt[4];
    QPoint rgb[3];
    int tailleImageZone[4];
    IplImage * imageZone[4];
    for(i=0; i < 3; i++){
        _Zone[i]=_Display->_Element[i]->rectangle();
        _Zone[i].setRect((int)(_Zone[i].left()*ratio_X), (int)(_Zone[i].top()*ratio_Y), (int)(_Zone[i].width()*ratio_X), (int)(_Zone[i].height()*ratio_Y));
        _Zone[i]=_Zone[i].normalized();
        tailleImageZone[i]=_Zone[i].height()*_Zone[i].width();
        imageZone[i] = cvCreateImage(cvSize( tailleImageZone[i],1), IPL_DEPTH_8U, 3);
        cvSet(imageZone[i],cvScalarAll(0),0);
        rgb[i]=QPoint(255,0);
        zoneInt[i]=0;
    }
    tailleImageZone[3]=_QImage->width()*_QImage->height()-(tailleImageZone[0]+tailleImageZone[1]+tailleImageZone[2]);
    imageZone[3] = cvCreateImage(cvSize(tailleImageZone[3],1), IPL_DEPTH_8U, 3);
    cvSet(imageZone[3],cvScalarAll(0),0);
    zoneInt[3]=0;

    // Optimisation des bornes du seuil, creation des zones (opration faites sur la QImage)
    bool zone[3];
    QColor couleur;
    unsigned char * image;
    for(y = 0; y < _QImage->height(); y++) { for(x = 0; x < _QImage->width(); x++) {
        couleur.setRgb(_QImage->pixel(x,y));
        if( (zone[0]=_Zone[0].contains(x,y)) || (zone[1]=_Zone[1].contains(x,y)) || (zone[2]=_Zone[2].contains(x,y)) ) {
            rgb[0].rx()= ( rgb[0].x() > couleur.red() ) ? couleur.red() : rgb[0].x();
            rgb[0].ry()= ( rgb[0].y() < couleur.red() ) ? couleur.red() : rgb[0].y();
            rgb[1].rx()= ( rgb[1].x() > couleur.green() ) ? couleur.green() : rgb[1].x();
            rgb[1].ry()= ( rgb[1].y() < couleur.green() ) ? couleur.green() : rgb[1].y();
            rgb[2].rx()= ( rgb[2].x() > couleur.blue() ) ? couleur.blue() : rgb[2].x();
            rgb[2].ry()= ( rgb[2].y() < couleur.blue() ) ? couleur.blue() : rgb[2].y();
            for(i = 0; i < 3; i++){
                if(zone[i]){
                    image = &((uchar*)(imageZone[i]->imageData))[zoneInt[i]*3];
                    image[2]=couleur.red();
                    image[1]=couleur.green();
                    image[0]=couleur.blue();
                    zoneInt[i]++;
                }
            }
        } else {
            image = &((uchar*)(imageZone[3]->imageData))[zoneInt[3]*3];
            image[2]=couleur.red();
            image[1]=couleur.green();
            image[0]=couleur.blue();
            zoneInt[3]++;
        }
    }}

    // Paramtre de l'algorithme gntique
    QPoint intervalPopulation, intervalIntermediaire;
    int populationTotale = 50;
    int populationElite = populationTotale*10/100;
    int populationMutee = populationTotale*30/100;
    int populationCroisee = populationTotale*60/100;
    int nombreIteration = 200;
    double pasProgression = 100.0/nombreIteration;

    QPoint intervalMutation[3];
    for(i=0; i<3; i++){
        y = (rgb[i].y()-rgb[i].x())/5;
        x = -y;
        intervalMutation[i] = QPoint(x,y);
    }

    // Cration des tableaux contenant les donnes
    Array<Gene,1> * tableauPopulation = new Array<Gene,1>(populationTotale);
    Array<Gene,1> * tableauIntermediaire = new Array<Gene,1>(populationTotale);
    Array<Gene,1> * tableauReference = 0;

    // Cration de la population de gne
    intervalPopulation = QPoint(0, populationTotale);
    generePopulationAleatoire(*tableauPopulation, intervalPopulation, rgb);

    // Algorithme genetique
    i = 0;
    Gene * seuil;
    IplImage * ilpimage;
    double zoneFloat[4];
    while(i < nombreIteration){
        j= (i==0) ? j = 0 : populationElite;
        while(j < populationTotale){
            // Rcupration du gene
            seuil = &(*tableauPopulation)(j);

            // Calcul du nombre de gnes prsent dans les zones
            for(x = 0; x < 4; x++){
                ilpimage = cvCreateImage(cvGetSize(imageZone[x]), IPL_DEPTH_8U, 1);
                cvInRangeS(imageZone[x],CV_RGB(seuil->rgb[0].x(),seuil->rgb[1].x(),seuil->rgb[2].x()),CV_RGB(seuil->rgb[0].y()+1,seuil->rgb[1].y()+1,seuil->rgb[2].y()+1),ilpimage);
                zoneFloat[x]=cvCountNonZero(ilpimage);
                cvReleaseImage(&ilpimage);
            }

            // Calcul de l'heuristique
            switch(_Heuristique){
                case HEU_0:
                    seuil->valeur=(zoneFloat[0]+zoneFloat[1]+zoneFloat[2])/(zoneFloat[3]+1);
                break;
                case HEU_1:
                    seuil->valeur=(zoneFloat[0]+zoneFloat[1]+zoneFloat[2])/(zoneFloat[3]+1) * (1.0/ sqrt(1.0/3.0*(pow(double(zoneFloat[0]/tailleImageZone[0]),2.0)+pow(double(zoneFloat[1]/tailleImageZone[1]),2.0)+pow(double(zoneFloat[2]/tailleImageZone[2]),2)) - pow(double(1.0/3.0*(zoneFloat[0]/tailleImageZone[0]+zoneFloat[1]/tailleImageZone[1]+zoneFloat[2]/tailleImageZone[2])),2.0)));
                    if(seuil->valeur!=seuil->valeur){
                        seuil->valeur=0;
                    }
                break;
                default:
                break;
            }

            // Incrmentation du compteur de boucle
            j++;
        }

        // Rcupration de l'interval contenant la population
        intervalPopulation = QPoint(0,populationTotale-1);

        // Tri des donnes
        Divers::triRapide(*tableauPopulation, intervalPopulation);

        // Affichage
        affichage(*tableauPopulation, intervalPopulation);

        i++;
        emit rafraichitProgression((int) (pasProgression*i) );
        if(nombreIteration!=i){

            // Sauvegarde des elites
            elite(*tableauPopulation, *tableauIntermediaire, intervalPopulation);

            // Mutation d'une partie de la population
            intervalIntermediaire = QPoint(populationElite,populationElite+populationMutee);
            mutation(*tableauPopulation, *tableauIntermediaire, intervalPopulation, intervalIntermediaire, intervalMutation, rgb);

            // Croisement d'une partie de la population
            intervalIntermediaire = QPoint(populationElite+populationMutee,populationTotale);
            croisement(*tableauPopulation, *tableauIntermediaire, intervalPopulation, intervalIntermediaire, rgb);

            // Permutation des tableaux
            tableauReference = tableauPopulation;
            tableauPopulation = tableauIntermediaire;
            tableauIntermediaire = tableauReference;
        }
    }
    // Sauvegarde du meilleur gne
    _Gene=(*tableauPopulation)(0);

    // Cration de l'image seuill
    genereImageSeuil((*tableauPopulation)(0));

    // Le calcul est termin
    emit calculTermine();
}
//
/*************************************************************************/
//
void Seuil::generePopulationAleatoire(Array<Gene,1>& tableauEntree, const QPoint& zoneEntree, const QPoint intervalRGB[]) {
    QPoint aleatoire;   // Bornes alatoires infrieures et suprieures
    for(int i = zoneEntree.x(); i < zoneEntree.y(); i++) {  // Parcourt du tableau dans la zone en paramtre
        for(int j = 0; j < 3; j++) {    // Parcourt des trois composantes de couleurs
            aleatoire.rx() = Divers::aleatoire(intervalRGB[j].x(), intervalRGB[j].y()); // Rcupration d'une borne
            aleatoire.ry() = Divers::aleatoire(intervalRGB[j].x(), intervalRGB[j].y()); // Rcupration d'une autre borne
            tableauEntree(i).rgb[j] = ( aleatoire.x()<aleatoire.y() ) ? QPoint(aleatoire.x(), aleatoire.y()) : QPoint(aleatoire.y(), aleatoire.x()); // Stockage de la borne infrieure et suprieure
        }
        tableauEntree(i).valeur = 0.0;  // Valeur du gne
        tableauEntree(i).etat = NORMAL; // Etat du gne
    }
}
//
/*************************************************************************/
//
void Seuil::elite(Array<Gene,1>& tableauEntree, Array<Gene,1>& tableauSortie, const QPoint& zoneSortie){
    for(int i = zoneSortie.x(); i < zoneSortie.y(); i++){
        tableauSortie(i) = tableauEntree(i);
        tableauSortie(i).etat = ELITE;
    }

    //tableauSortie(Range(zone.x(),zone.y()))=tableauEntree(Range(zone.x(),zone.y()));
}
//
/*************************************************************************/
//
void Seuil::mutation(Array<Gene,1>& tableauEntree, Array<Gene,1>& tableauSortie, const QPoint& zoneEntree, const QPoint& zoneSortie, const QPoint zoneAleatoire[], const QPoint intervalRGB[]){
    QPoint aleatoire;   // Bornes alatoires infrieures et suprieures
    int geneAleatoire;  // Numro du gne alatoire slctionn
    for(int i = zoneSortie.x(); i < zoneSortie.y(); i++){ // Parcourt du tableau de sortie avec la zone en paramtre
        geneAleatoire = Divers::aleatoire(zoneEntree.x(), zoneEntree.y());    // Acquisition d'un gne alatoire
        for(int j = 0; j < 3; j++){  // Parcourt des trois composantes de couleurs
            aleatoire.rx() = Divers::aleatoire(zoneAleatoire[j].x(), zoneAleatoire[j].y());   // Gnration d'une valeur alatoire (borne infrieure)
            aleatoire.ry() = Divers::aleatoire(zoneAleatoire[j].x(), zoneAleatoire[j].y());   // Gnration d'une valeur alatoire (borne suprieure)

            tableauSortie(i).rgb[j].rx() = tableauEntree(geneAleatoire).rgb[j].x() + aleatoire.x(); // Stockage d'une borne d'une composante du gne dans le tableau de sortie + mutation
            // Technique du rebond (il faut rester dans l'intervalle admissible)
            if(tableauSortie(i).rgb[j].x()>intervalRGB[j].y()) { tableauSortie(i).rgb[j].rx()=intervalRGB[j].y()*2-tableauSortie(i).rgb[j].x(); } else
            if(tableauSortie(i).rgb[j].x()<intervalRGB[j].x()) { tableauSortie(i).rgb[j].rx()=intervalRGB[j].x()*2-tableauSortie(i).rgb[j].x(); }

            tableauSortie(i).rgb[j].ry() = tableauEntree(geneAleatoire).rgb[j].y() + aleatoire.y(); // Stockage d'une borne d'une composante du gne dans le tableau de sortie + mutation
            // Technique du rebond (il faut rester dans l'intervalle admissible)
            if(tableauSortie(i).rgb[j].y()>intervalRGB[j].y()) { tableauSortie(i).rgb[j].ry()=intervalRGB[j].y()*2-tableauSortie(i).rgb[j].y(); } else
            if(tableauSortie(i).rgb[j].y()<intervalRGB[j].x()) { tableauSortie(i).rgb[j].ry()=intervalRGB[j].x()*2-tableauSortie(i).rgb[j].y(); }

            // Permutation des bornes ?
            if( tableauSortie(i).rgb[j].x()>tableauSortie(i).rgb[j].y() ){ tableauSortie(i).rgb[j] =  QPoint(tableauSortie(i).rgb[j].y(), tableauSortie(i).rgb[j].x()); }

        /*
            if( (tableauSortie(i).rgb[j].x()<0) || (tableauSortie(i).rgb[j].y()<0) ){
                std::cout << "Aleatoire : " << aleatoire.x() << "   " << aleatoire.y() << std::endl;
                std::cout << "RGB : " << tableauEntree(i).rgb[j].x() << "   " << tableauEntree(i).rgb[j].y() << std::endl;
                std::cout << "RGB : " << tableauSortie(i).rgb[j].x() << "   " << tableauSortie(i).rgb[j].y() << std::endl;
                std::cout << "ERREUR mutation" << std::endl;
                QTest::qSleep(4000);
            }
        */

        }

        tableauSortie(i).valeur = 0.0;  // Valeur du gne
        tableauSortie(i).etat = MUTEE;  // Etat du gne
    }
}
//
/*************************************************************************/
//
void Seuil::croisement(Array<Gene,1>& tableauEntree, Array<Gene,1>& tableauSortie, const QPoint& zoneEntree, const QPoint& zoneSortie, const QPoint intervalRGB[]){
    int geneAleatoire1, geneAleatoire2;   // Dclaration des variables permettant de rcuperer les gnes
    float a = 1.2;  // Le nombre magique
    for(int i = zoneSortie.x(); i < zoneSortie.y(); i++) { // Parcourt du tableau de sortie avec la zone en paramtre
        geneAleatoire1 = Divers::aleatoire(zoneEntree.x(), zoneEntree.y());   // Acquisition d'un gne alatoire
        geneAleatoire2 = Divers::aleatoire(zoneEntree.x(), zoneEntree.y());   // Acquisition d'un second gne alatoire
        if(tableauEntree(geneAleatoire1).valeur>tableauEntree(geneAleatoire2).valeur){  // Croisement selon la qualit du gne
            for(int j = 0; j < 3; j++){ // Parcourt des trois composantes de couleurs
                // Croisement de la borne infrieure
                tableauSortie(i).rgb[j].rx() = (int)(tableauEntree(geneAleatoire1).rgb[j].x()*a + tableauEntree(geneAleatoire2).rgb[j].x()*(1-a));
                if(tableauSortie(i).rgb[j].x()>intervalRGB[j].y()) { tableauSortie(i).rgb[j].rx()=intervalRGB[j].y()*2-tableauSortie(i).rgb[j].x(); } else
                if(tableauSortie(i).rgb[j].x()<intervalRGB[j].x()) { tableauSortie(i).rgb[j].rx()=intervalRGB[j].x()*2-tableauSortie(i).rgb[j].x(); }
                // Croisement de la borne suprieure
                tableauSortie(i).rgb[j].ry() = (int)(tableauEntree(geneAleatoire1).rgb[j].y()*a + tableauEntree(geneAleatoire2).rgb[j].y()*(1-a));
                if(tableauSortie(i).rgb[j].y()>intervalRGB[j].y()) { tableauSortie(i).rgb[j].ry()=intervalRGB[j].y()*2-tableauSortie(i).rgb[j].y(); } else
                if(tableauSortie(i).rgb[j].y()<intervalRGB[j].x()) { tableauSortie(i).rgb[j].ry()=intervalRGB[j].x()*2-tableauSortie(i).rgb[j].y(); }
                // Permutation des bornes ?
                if( tableauSortie(i).rgb[j].x()>tableauSortie(i).rgb[j].y() ){ tableauSortie(i).rgb[j] =  QPoint(tableauSortie(i).rgb[j].y(), tableauSortie(i).rgb[j].x()); }
            }
        } else {
            for(int j = 0; j < 3; j++){
                tableauSortie(i).rgb[j].rx() = (int)(tableauEntree(geneAleatoire2).rgb[j].x()*a + tableauEntree(geneAleatoire1).rgb[j].x()*(1-a));
                if(tableauSortie(i).rgb[j].x()>intervalRGB[j].y()) { tableauSortie(i).rgb[j].rx()=intervalRGB[j].y()*2-tableauSortie(i).rgb[j].x(); } else
                if(tableauSortie(i).rgb[j].x()<intervalRGB[j].x()) { tableauSortie(i).rgb[j].rx()=intervalRGB[j].x()*2-tableauSortie(i).rgb[j].x(); }
                tableauSortie(i).rgb[j].ry() = (int)(tableauEntree(geneAleatoire2).rgb[j].y()*a + tableauEntree(geneAleatoire1).rgb[j].y()*(1-a));
                if(tableauSortie(i).rgb[j].y()>intervalRGB[j].y()) { tableauSortie(i).rgb[j].ry()=intervalRGB[j].y()*2-tableauSortie(i).rgb[j].y(); } else
                if(tableauSortie(i).rgb[j].y()<intervalRGB[j].x()) { tableauSortie(i).rgb[j].ry()=intervalRGB[j].x()*2-tableauSortie(i).rgb[j].y(); }
                // Permutation des bornes ?
                if( tableauSortie(i).rgb[j].x()>tableauSortie(i).rgb[j].y() ){ tableauSortie(i).rgb[j] =  QPoint(tableauSortie(i).rgb[j].y(), tableauSortie(i).rgb[j].x()); }
            }
        }
        /*
        if( (tableauSortie(i).rgb[0].x()<0) || (tableauSortie(i).rgb[0].x()<0) || (tableauSortie(i).rgb[1].x()<0) || (tableauSortie(i).rgb[1].y()<0) || (tableauSortie(i).rgb[2].x()<0) || (tableauSortie(i).rgb[2].y()<0) ){
            std::cout << "ERREUR croisement" << std::endl;
            QTest::qSleep(4000);
        }
        */
        tableauSortie(i).valeur = 0.0;  // Valeur du gne
        tableauSortie(i).etat = CROISEE; // Etat du gne
    }
}
//
/*************************************************************************/
//
void Seuil::affichage(Array<Gene,1>& tableauEntree, const QPoint& zoneEntree){
    for(int i =  zoneEntree.y(); i >= zoneEntree.x(); i--){
        std::cout << "**************************" << std::endl;
        std::cout << "Numero du gene dans le tableau : " << i << std::endl;
        std::cout << "RGB rouge : (" << tableauEntree(i).rgb[0].x() << ", " << tableauEntree(i).rgb[0].y() << ")" << std::endl;
        std::cout << "RGB vert : (" << tableauEntree(i).rgb[1].x() << ", " << tableauEntree(i).rgb[1].y() << ")" << std::endl;
        std::cout << "RGB bleu : (" << tableauEntree(i).rgb[2].x() << ", " << tableauEntree(i).rgb[2].y() << ")" << std::endl;
        std::cout << "Valeur calculee : " << tableauEntree(i).valeur << std::endl;
        std::cout << "**************************" << std::endl;

    }
}
//
/*************************************************************************/
//
void Seuil::genereImageSeuil(const Gene& seuil){
    QColor pixel;
    for(int y = 0; y < _ImageSeuil.height(); y++) { for(int x = 0; x < _ImageSeuil.width(); x++) {
        pixel = _ImageSeuil.pixel(x,y);
        if(!Divers::dansSeuil(pixel,seuil)) {
            QColor color(0,0,0);
            _ImageSeuil.setPixel(x,y,color.rgb());
        } else {
            QColor color(255,255,255);
            _ImageSeuil.setPixel(x,y,color.rgb());
        }
     }}
}
//
/*************************************************************************/
//
}
//
